{% extends "admin/change_list.html" %}
{% load i18n %}


{% block object-tools-items %}
    {% if not is_popup %}
        <li><a href="{{recoverlist_url}}" class="recoverlink">{% blocktrans with cl.opts.verbose_name_plural|escape as name %}Recover deleted {{name}}{% endblocktrans %}</a></li>
    {% endif %}
    {{block.super}}
{% endblock %}